/***************************************************************************
                          ranpw.cpp  -  description
                             -------------------
    begin                : Tue Apr 13 14:40:09 EDT 2004
    copyright            : (C) 2004 by John S. Riley
    email                : dsbscience@cetlink.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// INCLUDES /////////////////////////////////////////////////// 
#include <ranpw.h>
#include <qstring.h>
#include <stdlib.h>
#include <time.h>

// DEFINES ///////////////////////////////////////////////////
#define ALPHA_ONLY_MIXED  18
#define ALPHA_ONLY_UPPER  66
#define ALPHA_ONLY_LOWER  34
#define NUMERIC_ONLY       4
#define ALPHANUMERIC_MIXED   16
#define ALPHANUMERIC_UPPER   64
#define ALPHANUMERIC_LOWER   32
#define CHARSET_NUMERIC       0
#define CHARSET_ALPHA_UPPER   1
#define CHARSET_ALPHA_LOWER   2

// FUNCTIONS /////////////////////////////////////////////////
RanPW::RanPW(QWidget *parent, const char *name) : RanPWDlg(parent, name)
{

  // seed the random number generator
  srand((unsigned int)time((time_t *)NULL));  

}

RanPW::~RanPW()
{
}

// button1 click slot implementation
void RanPW::Button1Clicked()
{

  //local declarations
  int OptFlag;            // alphanumeric option flag
  uint CharCounter;       // iteration counter to build char string
  uint NumberChar;        // number of characters to generate
  int NumberPW;           // number of passwords to generate
  int PwChar;             // generated random ascii value
  int CharSet;            // character set to select a random

                          // character from
                        
  // character strings
  QChar PwString[20];   // generated password
    
  // set alphanumeric option flag according to selected option
  if ( this->Option1->isChecked() ) OptFlag = 0;
  if ( this->Option2->isChecked() ) OptFlag = 2;
  if ( this->Option3->isChecked() ) OptFlag = 4;

  // modify flag if alpha or alpha numeric to include case
  if ( OptFlag < 4 )
  {
    if ( this->Option4->isChecked() )
    {
      OptFlag = OptFlag | 16;
    }
    if ( this->Option5->isChecked() )
    {
      OptFlag = OptFlag | 32;
    }
    if ( this->Option6->isChecked() )
    {
      OptFlag = OptFlag | 64;
    }
  }

  // initialize text box
  this->pwout->setText("");

  // get number of characters from ui
  NumberChar = this->numchar->text().toInt();

  // don't let number of characters exceed 20
  if (NumberChar > 20) NumberChar = 20;

  // get number of passwords from ui
  NumberPW = this->numpw->text().toInt();

  // don't let number of pw's exceed 1000
  if (NumberPW > 1000) NumberPW = 1000;

  //generate random text passwords
  for(int PwCounter=0; PwCounter < NumberPW; ++PwCounter)
  {
  
  for (CharCounter=0; CharCounter < NumberChar; ++CharCounter){

    switch (OptFlag) {

      // numeric only
      case NUMERIC_ONLY:
        PwChar = rand()%10 + 48;
        break;

      // alpha numeric with both upper and lower cases
      case ALPHANUMERIC_MIXED:

        // get a random character set
        CharSet = rand()%3;

        // assign a character from the char set
        switch (CharSet)
        {
          case CHARSET_NUMERIC:
            PwChar = rand()%10 + 48;
            break;

          case CHARSET_ALPHA_UPPER:
            PwChar = rand()%26 + 65;
            break;

          case CHARSET_ALPHA_LOWER:
            PwChar = rand()%26 + 97;
            break;
        }
        break;

      // alpha numeric with lower case only
      case ALPHANUMERIC_LOWER:

        //get a random character set
        CharSet = rand()%2;
        if (CharSet==1) ++CharSet;

        // assign a character from char set
        switch (CharSet)
        {
          case CHARSET_NUMERIC:
            PwChar = rand()%10 + 48;
            break;

          case CHARSET_ALPHA_LOWER:
            PwChar = rand()%26 + 97;
            break;

        }
        break;

      // alpha numeric with upper case only
      case ALPHANUMERIC_UPPER:

        // get random character set
        CharSet = rand()%2;

        // assign character from char set
        switch (CharSet)
        {
          case CHARSET_NUMERIC:
            PwChar = rand()%10 + 48;
            break;

          case CHARSET_ALPHA_UPPER:
            PwChar = rand()%26 + 65;
            break;

        }
        break;

      // alpha only with both upper and lower case
      case ALPHA_ONLY_MIXED:

        // get random character set
        CharSet = rand()%2;
        ++CharSet;

        // assign character from char set
        switch (CharSet)
        {
          case CHARSET_ALPHA_UPPER:
            PwChar = rand()%26 + 65;
            break;

          case CHARSET_ALPHA_LOWER:
            PwChar = rand()%26 + 97;
            break;
        }
        break;
      
      // alpha only with lower case only
      case ALPHA_ONLY_LOWER:
        PwChar = rand()%26 + 97;
        break;

      // alpha only with upper case only
      case ALPHA_ONLY_UPPER:
        PwChar = rand()%26 + 65;
        break;

    }

    PwString[CharCounter] = PwChar;

  }
        
  this->pwout->append ( QString(PwString, NumberChar) );

  }
    
} // end button1 click slot implementation
